#!/usr/bin/env bash

declare    IFS=$'\n' zeroPad="0000" startPosOutput direction=1 gridCount=0
declare -a grid gridRotated terrainLayer
declare -i i x y z mapWidth=55 mapHeight=50 orient=0 startPosCount=0 riverY riverX tilesPerPlayer=256 gridCount=0

# Function RotateGrid (clockwise, 1/4 turn)
RotateGrid() {
	local -i i z
	declare -a gridRotated
	if [ $((${RANDOM}%2)) -eq 1 -o "${1} " = "cw " ] ; then
			# Rotate 1/4 turn clockwise
			direction="cw"
			for ((i=0,z=0;i<5;i++,z++)) ; do
				gridRotated[${i}]="${grid[4]:${z}:1}${grid[3]:${z}:1}${grid[2]:${z}:1}${grid[1]:${z}:1}${grid[0]:${z}:1}"
			done
		else
			# Rotate 1/4 turn counter-clockwise
			direction="ccw"
			for ((i=0,z=4;i<5;i++,z--)) ; do
				gridRotated[${i}]="${grid[0]:${z}:1}${grid[1]:${z}:1}${grid[2]:${z}:1}${grid[3]:${z}:1}${grid[4]:${z}:1}"
			done
	fi
	grid=(${gridRotated[*]})
}

CheckStartPosition() {
	if [ "${terrainLayer[${startPosY}]:${startPosX}:1}" = " " -o "${terrainLayer[${startPosY}]:${startPosX}:1}" = ":" -o "${terrainLayer[${startPosY}]:${startPosX}:1}" = "+" ] ; then
		startPosX=$(( ${startPosX} + 1 ))
		CheckStartPosition
	fi
}


# Check map width/height minimums
if [ ${mapWidth} -lt 16 -o ${mapHeight} -lt 16 ] ; then
	echo "FreeCiv minimum map width/height is 16"
	exit 1
fi

# Constrain map height/width to multiples of 5 (the size of the grid pattern)
mapWidth=$(( (${mapWidth} / 5) * 5 ))
mapHeight=$(( (${mapHeight} / 5) * 5 ))

# Create the other map layer arrays
riverLayer[0]=$(temp=$(printf "%*s" ${mapWidth}) ; echo "${temp// /0}")
e00Layer[0]="${riverLayer[0]}"
e01Layer[0]="${riverLayer[0]}"
e02Layer[0]="${riverLayer[0]}"
resLayer[0]=$(printf "%*s" ${mapWidth})

for ((i=1;i<${mapHeight};i++)) ; do
	e00Layer[${i}]="${riverLayer[0]}"
	e01Layer[${i}]="${riverLayer[0]}"
	e02Layer[${i}]="${riverLayer[0]}"
	riverLayer[${i}]="${riverLayer[0]}"
	resLayer[${i}]="${resLayer[0]}"
done


# 5x5 grid of terrain tiles to place
grid[0]="phpp+"
grid[1]="ppppp"
grid[2]="pfghg"
grid[3]="ggggg"
grid[4]="gfgfh"


# Start on first row, work across, then back and down like a typewriter
for ((y=0;y<${mapHeight};y+=5)) ; do
	for ((x=0;x<${mapWidth};x+=5)) ; do

		RotateGrid

		# 50/50 chance of a second rotation in same direction
		[ $((${RANDOM}%2)) -eq 0 ] && RotateGrid "${direction}"

		# Add rotated grid pattern to map
		for ((z=0;z<5;z++)) ; do
			terrainLayer[$((${y}+${z}))]="${terrainLayer[$((${y}+${z}))]}${grid[${z}]}"
		done

		gridCount=$((gridCount+1))

		# Add start position for center tile
		# If desired, start positions can be added here as x+2, y+2 just like the river coords below.
		# Doing so results in 1 startPos per grid which is -many- more, BUT, when playing with fewer
		# players than start positions, the game doesn't select from among the available startPos for
		# fair/even spacing -- and fairness is the entire aim of this script! So I switched methods,
		# and the new startPos routine is further below, just before the "output the map" section.

		# Add river tile for Hoover Dam
		riverY=$((${y}+2))
		riverX=$((${x}+2))
		riverLayer[${riverY}]="${riverLayer[${riverY}]:0:${riverX}}2${riverLayer[${riverY}]:$((${riverX}+1))}"
	done
done



# Generate start positions fairly / evenly spaced out
# Results in far fewer start positions then # of patterns
totalLandTiles=$(( ${gridCount} * 21 ))
players=$(( ${totalLandTiles} / ${tilesPerPlayer} ))
areaSizeSqrt=$(echo "scale=0; sqrt(${tilesPerPlayer})" | bc)

areasAcross=$(( ${mapWidth} / ${areaSizeSqrt} ))
areasHigh=$(( ${mapHeight} / ${areaSizeSqrt} ))

for ((i=0;i<$((${mapHeight}-${areaSizeSqrt}));i+=${areaSizeSqrt})) ; do
	for ((j=0;j<$((${mapWidth}-${areaSizeSqrt}));j+=${areaSizeSqrt})) ; do
		startPosX=$(( ${i} + ( ${areaSizeSqrt} / 2 ) ))
		startPosY=$(( ${j} + ( ${areaSizeSqrt} / 2 ) ))
		CheckStartPosition
		startPosOutput="${startPosOutput}${startPosX},${startPosY},FALSE,\"\"${IFS}"
		startPosCount=$(( ${startPosCount} + 1 ))
	done
done



# Output the map for copy/paste to a FreeCiv ".sav" file

# t (terrain) layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "t${zeroPad:${#i}}${i}=\"${terrainLayer[${i}]}\""
done

# Start positions list
echo "startpos_count=${startPosCount}"
echo "startpos={\"x\",\"y\",\"exclude\",\"nations\""
echo -n "${startPosOutput}"
echo '}'

# e00_ Irrigation, mine, pollution, oil wells layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "e00_${zeroPad:${#i}}${i}=\"${e00Layer[${i}]}\""
done

# e01_ Hut, farmland, fallout, fortress layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "e01_${zeroPad:${#i}}${i}=\"${e01Layer[${i}]}\""
done

# e02_ Rail, airbase, buoy, ruins layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "e02_${zeroPad:${#i}}${i}=\"${e02Layer[${i}]}\""
done

# e03_ River and road layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "e03_${zeroPad:${#i}}${i}=\"${riverLayer[${i}]}\""
done

# res (resources) layer
for ((i=0;i<${mapHeight};i++)) ; do
	echo "res${zeroPad:${#i}}${i}=\"${resLayer[${i}]}\""
done

echo
echo "#### Actual width/xsize=${mapWidth} height/ysize=${mapHeight}"
echo

exit 0
